<?php

/**
 * @file
 * Field module functionality for the Encrypted Files module.
 */

/**
 * Implements hook_field_create_instance().
 */
function encrypted_files_field_create_instance($instance) {
  // Make sure instances of the password-protected field use right formatter.
  if ($instance['field_name'] != 'encrypted_files_pass_protected') {
    return;
  }
  $instance['display'] = array(
    'default' => array(
      'type' => 'encrypted_files_protected',
    ),
  );
  field_update_instance($instance);
}

/**
 * Implements hook_field_widget_info().
 */
function encrypted_files_field_widget_info() {
  return array(
    // Widget for password-protecting a file.
    'encrypted_files_protected' => array(
      'label' => t('Password-Protected File'),
      'description' => t('Set up a file field to store encrypted,
        password-protected files.'),
      'field types' => array('file'),
      'behaviors' => array(
        'default value' => FIELD_BEHAVIOR_NONE,
      ),
      'settings' => array(
        'progress_indicator' => 'throbber',
      ),
    ),
  );
}

/**
 * Implements hook_field_widget_form().
 */
function encrypted_files_field_widget_form(&$form, &$form_state, $field,
  $instance, $langcode, $items, $delta, $element) {
  // Load File module's file widget, which we'll extend.
  $elements = file_field_widget_form($form, $form_state, $field,
    $instance, $langcode, $items, $delta, $element);
  $elements = $elements[0];
  // Add form elements for password-protected files, as long as there's no file
  // yet or the file remove button was clicked.
  $is_correct_field = ($instance['widget']['type'] ==
    'encrypted_files_protected');
  $remove_button_clicked = isset($form_state['triggering_element']['#name']) &&
    (strpos($form_state['triggering_element']['#name'],
      'remove_button') !== FALSE);
  if ($is_correct_field && (empty($items) || $remove_button_clicked)) {
    // Add inputs for password to use in encryption.
    $elements['encrypted_files_password'] = array(
      '#type' => 'password_confirm',
      '#title' => t('File Password'),
    );
  }
  // Add hidden fields to persist field info.
  if (!$remove_button_clicked) {
    $form['encrypted_files_field_name'] = array(
      '#type' => 'hidden',
      '#value' => $field['field_name'],
    );
    $form['encrypted_files_field_language'] = array(
      '#type' => 'hidden',
      '#value' => $langcode,
    );
  }
  // Set our process function.
  $elements['#process'][] = 'encrypted_files_process_password_file';

  return $elements;
}

/**
 * Implements hook_field_formatter_info().
 */
function encrypted_files_field_formatter_info() {
  return array(
    // Formatter to display a password-protected file.
    'encrypted_files_protected' => array(
      'label' => t('Password-Protected File'),
      'description' => t('Displays a form to download
        a password-protected file'),
      'field types' => array('file'),
    ),
  );
}

/**
 * Implements hook_field_formatter_view().
 */
function encrypted_files_field_formatter_view($entity_type, $entity, $field,
  $instance, $langcode, $items, $display) {
  $element = array();

  switch ($display['type']) {
    case 'encrypted_files_protected':
      foreach ($items as $delta => $item) {
        $file = (object) $item;
        $form_state = array(
          'build_info' => array(
            'args' => array($file),
          ),
        );
        $element[$delta] = array(
          '#prefix' => '<div class="encrypted-files-download clearfix">',
          'file_name' => array(
            '#prefix' => '<strong class="encrypted-files-file-title">',
            '#markup' => $file->filename,
            '#suffix' => '</strong>',
          ),
          'password_form' => drupal_build_form('encrypted_files_password_form',
            $form_state),
          '#suffix' => '</div><!-- /.encrypted-files-download -->',
        );
      }
      break;
  }

  return $element;
}

/**
 * Callback function that generates a password form.
 *
 * Its form enables a user to download a password-protected file.
 *
 * @param object $file
 *   Object representing the file available for download
 *
 * @return array
 *   Form array
 */
function encrypted_files_password_form($form, &$form_state, $file) {
  $module_path = drupal_get_path('module', 'encrypted_files');
  return array(
    '#attached' => array(
      'css' => array(
        $module_path . '/css/encrypted-files-download.css',
      ),
    ),
    // Elements.
    'password' => array(
      '#type' => 'password',
      '#title' => t('Password'),
      '#size' => 20,
      '#required' => TRUE,
      '#attributes' => array(
        'class' => array('encrypted-files-password-field'),
      ),
    ),
    'submit' => array(
      '#type' => 'submit',
      '#value' => t('Download file'),
      '#attributes' => array(
        'class' => array('encrypted-files-download-submit'),
      ),
    ),
    'file' => array(
      '#type' => 'value',
      '#value' => $file,
    ),
  );
}

/**
 * Callback function that decrypts a file using the submitted password.
 */
function encrypted_files_password_form_submit($form, &$form_state) {
  // Get password from form and pass it to the stream wrapper for decryption.
  $key_set = EncryptedStreamWrapper::setKey($form_state['values']['password']);
  if (!$key_set) {
    drupal_set_message(t('Error: Unable to use password as encryption key'),
      'error');
    return;
  }
  // Check that user has access to the file.
  $file = $form_state['values']['file'];
  if (!encrypted_files_can_access_file($file->uri)) {
    drupal_set_message(t('Error: You do not have permission
      to download this file'), 'error');
    return;
  }
  // Send file to user.
  encrypted_files_transfer_file($file->uri, NULL);
}

/**
 * Processes form elements for the password-protected widget.
 */
function encrypted_files_process_password_file($element, &$form_state, $form) {
  // Remove the ajax file "Upload" button, as our password-protected files don't
  // support ajax uploading yet.
  if (isset($element['upload_button'])) {
    unset($element['upload_button']);
  }

  // Redo the filename without a link, as clicking the link for a
  // password-protected file will trigger an error.
  if (isset($element['filename'])) {
    $element['filename']['#markup'] = '<span class="file">' .
      check_plain($element['#file']->filename) . '</span>';
  }

  return $element;
}
