<?php

/**
 * @file
 * Defines the EncryptedStreamWrapper class.
 */

/**
 * Handles encrypting and decrypting files written to
 * and read from Encrypted File's custom stream wrapper.
 *
 * Extends private file system's wrapper to keep files safe in private system.
 */
class EncryptedStreamWrapper extends DrupalPrivateStreamWrapper {

  /**
   * Instance URI as scheme://target.
   */
  protected $uri;

  /**
   * A generic resource handle.
   */
  public $handle = NULL;

  // Mode the file was opened with.
  protected $mode = NULL;

  // Key to use for encryption & decryption.
  protected $key = NULL;

  // Key submitted from another part of module.
  protected static $manualKey = NULL;

  // Important info about the file.
  protected static $fileInfo = array();

  // Checksum for the file to be downloaded.
  protected static $checksums = array();

  /**
   * Returns full path to Encrypted Files subdirectory within private directory.
   *
   * @return string
   *   Directory path
   */
  public function getDirectoryPath() {
    return parent::getDirectoryPath() . '/' . ENCRYPTED_FILES_DIRECTORY;
  }

  /**
   * Opens a temporary stream in memory to store the unencrypted file.
   *
   * File is stored temporarily while Drupal is reading and writing it.
   */
  public function stream_open($uri, $mode, $options, &$opened_path) {
    // Create Encrypted Files directory if it doesn't exist.
    $ef_directory = $this->getDirectoryPath();
    if (!file_exists($ef_directory)) {
      mkdir($ef_directory, 0755);
    }
    // Load encryption key.
    $this->loadKey();
    // Load resource location.
    $this->uri = $uri;
    $path = $this->getLocalPath();
    // Save the mode for later reference.
    $this->mode = $mode;
    // Load temp file as our handle.
    $this->handle = fopen('php://memory', 'w+b');
    // If file exists, decrypt and load it into memory.
    if (file_exists($path)) {
      $raw_file = file_get_contents($path);
      $decrypted_file = base64_decode(aes_decrypt($raw_file,
        FALSE, $this->key));
      self::setFileInfo($decrypted_file, $uri);
      // Verify checksum.
      if (isset(self::$checksums[$uri]) &&
          (self::$checksums[$uri] != self::$fileInfo[$uri]['hash'])) {
        throw new FileTransferException(t('Failed to decrypt
          the requested file'));
      }
      // Write to memory.
      fwrite($this->handle, $decrypted_file);
      rewind($this->handle);
    }
    // Set $opened_path.
    if ((bool) $this->handle && $options & STREAM_USE_PATH) {
      $opened_path = $path;
    }

    return (bool) $this->handle;
  }

  /**
   * Encrypts and writes the open file to disk, then closes the stream.
   */
  public function stream_close() {
    // If file mode we opened with is only for reading,
    // don't resave the file.
    if ((strpos($this->mode, 'r') !== FALSE) &&
        (strpos($this->mode, '+') === FALSE)) {
      fclose($this->handle);
      return;
    }
    // Encrypt file and save.
    rewind($this->handle);
    $file_contents = stream_get_contents($this->handle);
    file_put_contents($this->getLocalPath(),
      aes_encrypt(base64_encode($file_contents), FALSE, $this->key));
    // Store important file info.
    self::setFileInfo($file_contents, $this->uri);
    // Close handle and reset manual key.
    fclose($this->handle);
    self::$manualKey = NULL;
  }

  /**
   * Overrides getExternalUrl().
   *
   * @return string
   *   The URI of an encrypted file.
   */
  public function getExternalUrl() {
    $path = str_replace('\\', '/', $this->getTarget());
    return url('system/' . ENCRYPTED_FILES_DIRECTORY . '/' .
      $path, array('absolute' => TRUE));
  }

  /**
   * Allows other parts of the module to manually set the encryption key.
   *
   * NOTE: As a precaution, key can only be manually set once per file.
   *
   * @param string $key
   *   A key to use for encryption/decryption instead of the default one
   *
   * @return bool
   *   TRUE if key saved successfully, FALSE if not
   */
  public static function setKey($key) {
    if (isset(self::$manualKey)) {
      return FALSE;
    }
    self::$manualKey = $key;
    return TRUE;
  }

  /**
   * Returns hash and file size for the given file.
   *
   * @param string $uri
   *   The URI of the file to retrieve info for
   *
   * @return array|bool
   *   Associative array with above info, or false if not found
   */
  public static function getFileInfo($uri) {
    return isset(self::$fileInfo[$uri]) ? self::$fileInfo[$uri] : FALSE;
  }

  /**
   * Sets the checksum with which to verify the integrity of the decrypted file.
   *
   * @param string $uri
   *   The uri of the file
   * @param string $checksum
   *   Hashed checksum of the file in its unencrypted state
   */
  public static function setChecksum($uri, $checksum) {
    self::$checksums[$uri] = $checksum;
  }

  /**
   * Stores important info about the file we're operating on.
   *
   * @param string $content
   *   The content of the file
   * @param string $name
   *   The filename
   */
  protected static function setFileInfo($content, $name) {
    self::$fileInfo[$name] = array(
      'hash' => hash('sha256', $content),
      'size' => strlen($content),
    );
  }

  /**
   * Loads key used in encryption and decryption.
   */
  protected function loadKey() {
    // If key set manually, use it.
    if (isset(self::$manualKey)) {
      $this->key = self::$manualKey;
      return;
    }
    // Otherwise, use the default key.
    module_load_include('inc', 'encrypted_files', 'includes/encrypted_files');
    $keys = _encrypted_files_load_keys();
    // @todo Decide how to handle failing to load keys.
    $this->key = _encrypted_files_decrypt_key($keys[0], $keys[1]);
  }

}
